-----------------------------------------------------------------------------------------------
-- Client Lua Script for Clean_XP_Bar
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "Apollo"
require "GroupLib"
require "PlayerPathLib"
 
-----------------------------------------------------------------------------------------------
-- Clean_XP_Bar Module Definition
-----------------------------------------------------------------------------------------------
local Clean_XP_Bar = {} 
local xpTooltip = "this is a test tooltip!"
local knMaxLevel = 50
local tStats = 0
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Clean_XP_Bar:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function Clean_XP_Bar:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Clean_XP_Bar OnLoad
-----------------------------------------------------------------------------------------------
function Clean_XP_Bar:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Clean_XP_Bar.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Clean_XP_Bar OnDocLoaded
-----------------------------------------------------------------------------------------------
function Clean_XP_Bar:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Clean_XP_Bar_Container", nil, self)
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self) -- saving pos/r-size
		Apollo.RegisterEventHandler("UI_XPChanged", 				"OnXpChanged", self)
		Apollo.RegisterEventHandler("ElderPointsGained", 			"OnXpChanged", self)

		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(true, true)
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		
		self.timer = ApolloTimer.Create(3.0, true, "ForceXPcheck", self)

		-- Do additional Addon initialization here
		--self:UpdateTooltips()
		--self:RedrawXP()
		if GameLib.GetPlayerUnit() ~= nil then
			self:ForceXPcheck()
		end
	end
end

function Clean_XP_Bar:ForceXPcheck()
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	if GameLib.GetPlayerUnit() ~= nil then
		self:OnXpChanged()
	end
end

function Clean_XP_Bar:OnWindowManagementReady() -- saving pos/r-size
    Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Clean_XP_Bar"})
	--self:OnXpChanged()
end

-----------------------------------------------------------------------------------------------
-- Clean_XP_Bar Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

function Clean_XP_Bar:OnXpChanged()
local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	local tStats = unitPlayer:GetBasicStats()
	if tStats ~= nil then
		self:UpdateTooltips()
		if tStats.nLevel == knMaxLevel then
			self:RedrawEP()
		else
			self:RedrawXP()
		end
	end
end

function Clean_XP_Bar:UpdateTooltips()
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	--Tooltip
	local tStats = unitPlayer:GetBasicStats()
	if tStats ~= nil then
		if tStats.nLevel == knMaxLevel then
			self.wndMain:SetTooltip("EP Tooltip Soon.TM")
		else
			local unitPlayer = GameLib.GetPlayerUnit()
			xpTooltip = self:ConfigureXPTooltip(unitPlayer)
			self.wndMain:SetTooltip(xpTooltip)
		end
	end
end

function Clean_XP_Bar:RedrawEP()
	
	local nCurrentEP = GetElderPoints()
	local nEPToAGem = GameLib.ElderPointsPerGem
	local nEPDailyMax = GameLib.ElderPointsDailyMax
	local nRestedEP = GetRestXp() 							-- amount of rested xp
	local nRestedEPPool = GetRestXpKillCreaturePool() 		-- amount of rested xp remaining from creature kills
	
	if not nCurrentEP or not nEPToAGem or not nEPDailyMax or not nRestedEP then
		return
	end
	
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarForm"):SetMax(nEPToAGem)
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarForm"):SetProgress(nCurrentEP)

end

function Clean_XP_Bar:RedrawXP()
	local nCurrentXP = GetXp() - GetXpToCurrentLevel()
	local nNeededXP = GetXpToNextLevel()
	local nRestedXP = GetRestXp()
	local nRestedXPPool = GetRestXpKillCreaturePool() 	
	
	if not nCurrentXP or not nNeededXP or not nNeededXP or not nRestedXP then
		return
	end
	
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarForm"):SetMax(nNeededXP)
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarForm"):SetProgress(nCurrentXP)
	
	-- Rest Bar and Goal (where it ends)
	--wndRestXPBarFill:SetMax(nNeededXP)
	--wndRestXPBarFill:Show(nRestedXP and nRestedXP > 0)
	if nRestedXP and nRestedXP > 0 then
		--wndRestXPBarFill:SetProgress(math.min(nNeededXP, nCurrentXP + nRestedXP))
	end

	local bShowRestXPGoal = nRestedXP and nRestedXPPool and nRestedXP > 0 and nRestedXPPool > 0
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarRested"):SetMax(nNeededXP)
	self.wndMain:FindChild("XPBarContainer:Clean_XP_BarRested"):Show(bShowRestXPGoal)
	if bShowRestXPGoal then
		self.wndMain:FindChild("XPBarContainer:Clean_XP_BarRested"):SetProgress(math.min(nNeededXP, nCurrentXP + nRestedXPPool))
	end
	
	
	
	
	return math.min(99.9, nCurrentXP / nNeededXP * 100)
end

function Clean_XP_Bar:ConfigureXPTooltip(unitPlayer)
	local nCurrentXP = GetXp() - GetXpToCurrentLevel() 		-- current amount of xp into the current level
	local nNeededXP = GetXpToNextLevel() 					-- total amount needed to move through current level
	local nRestedXP = GetRestXp() 							-- amount of rested xp
	local nRestedXPPool = GetRestXpKillCreaturePool() 		-- amount of rested xp remaining from creature kills

	if not nCurrentXP or not nNeededXP or not nNeededXP or not nRestedXP then
		return
	end

	local strTooltip = string.format("<P Font=\"CRB_InterfaceSmall_O\">%s</P>", String_GetWeaselString(Apollo.GetString("Base_XPValue"), nCurrentXP, nNeededXP, nCurrentXP / nNeededXP * 100))
	if nRestedXP > 0 then
		local strRestLineOne = String_GetWeaselString(Apollo.GetString("Base_XPRested"), nRestedXP, nRestedXP / nNeededXP * 100)
		strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, strRestLineOne)

		if nCurrentXP + nRestedXPPool > nNeededXP then
			strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, Apollo.GetString("Base_XPRestedEndsAfterLevelTooltip"))
		else
			local strRestLineTwo = String_GetWeaselString(Apollo.GetString("Base_XPRestedPoolTooltip"), nRestedXPPool, ((nRestedXPPool + nCurrentXP)  / nNeededXP) * 100)
			strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, strRestLineTwo)
		end
	end
	
	strTooltip = string.format("<P Font=\"CRB_InterfaceSmall_O\">%s%s</P>%s", Apollo.GetString("CRB_Level_"), unitPlayer:GetLevel(), strTooltip)
	
	return strTooltip
end

-----------------------------------------------------------------------------------------------
-- Clean_XP_BarForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked

--function Clean_XP_Bar:OnOK()
--	self.wndMain:Close() -- hide the window
--end

-- when the Cancel button is clicked
--function Clean_XP_Bar:OnCancel()
--	self.wndMain:Close() -- hide the window
--end


-----------------------------------------------------------------------------------------------
-- Clean_XP_Bar Instance
-----------------------------------------------------------------------------------------------
local Clean_XP_BarInst = Clean_XP_Bar:new()
Clean_XP_BarInst:Init()
